library geiger_localstorage;

import 'dart:async';

class ByteStream {

  static final Stream<List<int>> empty = Stream.empty();
  late Stream<List<int>> _stream;
  final List<int> _currentList = [];
  bool _closed = false;

  ByteStream([Stream<List<int>>? stream, List<int>? buffer]) {
    _stream = stream ?? Stream.empty();
    if (stream != null) {
      _stream.listen(
              (event) {
            _currentList.addAll(event);
          },
          onDone: () {
            _closed = true;
          });
    } else {
      _closed = true;
    }
    if (buffer != null) {
      _currentList.addAll(buffer);
    }
  }

  Future<List<int>> get bytes async {
    List<int> ret = [];
    ret.addAll(_currentList);
    List<List<int>> lists = await _stream.toList();
    for (var l in lists) {
      ret.addAll(l);
    }
    return ret;
  }

  Future<List<int>> peekBytes(int num) async {
    while(_currentList.length<num && !closed) {
      _currentList.addAll(await _stream.single);
    }
    return _currentList.take(num).toList();
  }

  bool get closed {
    return _closed;
  }

  Future<int> pop() async {
    if (_currentList.isEmpty && !closed) {
      // TODO: wait until data is available (busy waiting... how sad...)
      while (await Future<bool>.delayed(
          const Duration(milliseconds: 100), () => _currentList.isNotEmpty) &&
          !closed) {}
    }
    return _currentList.removeAt(0);
  }

}