library geiger_localstorage;

import 'byte_sink.dart';
import 'byte_stream.dart';

/// <p>Serializer interface for the serialization of value related objects.</p>
abstract class Serializer {
  /// <p>Dummy static serializer.</p>
  /// <p>Must be overridden by the respective implementing class.</p>
  /// @param in The input byte stream to be used
  /// @return the object parsed from the input stream by the respective class
  /// @throws IOException if not overridden or reached unexpectedly the end of stream
  static Serializer fromByteArrayStream(ByteStream in_) {
    throw Exception('Not implemented... should be overloaded');
  }

  /// <p>Writes the current object to the output stream.</p>
  /// @param out the output stream receiving the object
  /// @throws IOException if  an exception occurs while writing to the stream
  void toByteArrayStream(ByteSink out);

  /// Convenience class to serialize to a bytearray.
  /// @param obj the object to serialize
  /// @return byteArray representing the object
  static Future<List<int>> toByteArray(Serializer obj) async {
    ByteSink out = ByteSink();
    obj.toByteArrayStream(out);
    out.close();
    return out.bytes;
  }

  /// Convenience Class to deserialize using byte array.
  /// @param buf the byte array to deserialize
  /// @return Serializer
  static Serializer fromByteArray(List<int> buf) {
    try {
      return fromByteArrayStream(ByteStream(null,buf));
    } on UnimplementedError catch (e, s) {
      print("StackTrace $s");
      rethrow;
    }
  }
}
