import 'dart:async';

import 'package:geiger_localstorage/geiger_localstorage.dart';

class SerializedException with Exception implements Serializer {
  static const int serialversionUID = 721364991234;

  String _name = '';
  String _message = '';
  SerializedException? _rootCause;
  StackTrace? _stackTrace;

  SerializedException(
      [Exception? exception, StackTrace? stacktrace, Exception? rootCause]) {
    if (exception != null) {
      _name = exception.toString();
    }
    _stackTrace = stacktrace;
  }

  SerializedException.fromSpec(
      this._name, this._message, this._stackTrace, this._rootCause);

  set rootCause(Exception? rootCause) {
    if (rootCause != null) {
      if (rootCause.runtimeType != SerializedException) {
        _rootCause = SerializedException(rootCause);
      } else {
        _rootCause = rootCause as SerializedException;
      }
    } else {
      _rootCause = null;
    }
  }

  /// public SerializedException(String exceptionName, String message, StackTraceElement[] stacktrace,
  /// Throwable cause) {
  /// super(message, cause);
  /// this.exceptionName = exceptionName;
  /// this.message = message;
  /// setStackTrace(stacktrace);
  /// }
  ///
  get message {
    return _message;
  }

  @override
  void toByteArrayStream(ByteSink out) {
    SerializerHelper.writeLong(out, serialversionUID);
    SerializerHelper.writeString(out, _name);
    SerializerHelper.writeString(out, _message);
    if (_stackTrace != null) {
      SerializerHelper.writeInt(out, 1);
      SerializerHelper.writeString(out, _stackTrace.toString());
    } else {
      SerializerHelper.writeInt(out, 0);
    }
    if (_rootCause != null) {
      SerializerHelper.writeInt(out, 1);
      _rootCause!.toByteArrayStream(out);
    } else {
      SerializerHelper.writeInt(out, 0);
    }
    SerializerHelper.writeLong(out, serialversionUID);
  }

  static Future<SerializedException> fromByteArrayStream(ByteStream in_) async {
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('cannot cast');
    }
    String _name = await SerializerHelper.readString(in_) ?? 'UNKNOWN TEXT';
    String _message =
        await SerializerHelper.readString(in_) ?? 'UNKNOWN MESSAGE';

    StackTrace ste =
        StackTrace.fromString(await SerializerHelper.readString(in_) ?? '');

    SerializedException? cause;
    if (await SerializerHelper.readInt(in_) == 1) {
      cause = await SerializedException.fromByteArrayStream(in_);
    }
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('cannot cast');
    }
    return SerializedException.fromSpec(_name, _message, ste, cause);
  }

  @override
  String toString() {
    String ret = '$_name:$_message';
    if(_stackTrace!=null) {
      ret+=' at \n${_stackTrace.toString()}';
    }
    if(_rootCause!=null) {
      ret+='\nRoot cause:\n${_rootCause.toString()}';
    }
    return ret;
  }
}

class StorageException extends SerializedException {
  /// <p>Exception to be raised on any problems related to the local storage.</p>class StorageException implements IOException, Serializer {class StorageException extends SerializedException {
  static const int serialversionUID = 178324938;

  /// Creates a StorageException with message and root cause.
  ///
  /// A storageException contains always a descriptive string [_message] and may contain additional information such as another exception [_cause] and a [stackTrace].
  StorageException(String message,
      [StackTrace? stackTrace, Exception? rootCause, StackTrace? rcste]) {
    _message = message;
    _name='StorageException';
    _stackTrace = stackTrace;
    if(_stackTrace==null) {
      try {
        throw Exception();
      } catch (_,st) {
        _stackTrace=st;
      }
    }
    if(rootCause.runtimeType==SerializedException) {
      (rootCause as SerializedException)._stackTrace??=rcste;
      _rootCause=_rootCause as SerializedException;
    } else {
      if(rootCause!=null) {
        _rootCause = SerializedException(rootCause, rcste);
      }
    }
    rootCause = rootCause;
  }

  @override
  void toByteArrayStream(ByteSink out) {
    SerializerHelper.writeLong(out, serialversionUID);
    SerializerHelper.writeString(out, _message);
    if (_stackTrace == null) {
      SerializerHelper.writeInt(out, 0);
    } else {
      SerializerHelper.writeInt(out, 0);
      SerializerHelper.writeString(out, _stackTrace.toString());
    }
    if (_rootCause == null) {
      SerializerHelper.writeInt(out, 0);
    } else {
      SerializerHelper.writeInt(out, 1);
      _rootCause!.toByteArrayStream(out);
    }
    SerializerHelper.writeLong(out, serialversionUID);
  }

  /// <p>Static deserializer.</p>
  /// <p>Creates a storage exception from the stream.</p>
  ///
  /// @param in The input byte stream to be used
  /// @return the object parsed from the input stream by the respective class
  /// @throws IOException if not overridden or reached unexpectedly the end of stream
  static Future<StorageException> fromByteArrayStream(ByteStream in_) async {
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('cannot cast');
    }
    String txt = (await SerializerHelper.readString(in_)) ?? '';
    StackTrace ste =
        StackTrace.fromString(await SerializerHelper.readString(in_) ?? '');
    SerializedException? t;
    if (await SerializerHelper.readInt(in_) == 1) {
      t = await SerializedException.fromByteArrayStream(in_);
    }
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('cannot cast');
    }
    return StorageException(txt, ste, t);
  }
}
