import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:geiger_localstorage/src/serialization/byte_sink.dart';
import 'package:geiger_localstorage/src/serialization/byte_stream.dart';
import 'package:geiger_localstorage/src/visibility.dart';
import 'package:geiger_localstorage/src/db/data/node_implementation.dart';
import 'package:test/test.dart';

class NodeTest {}

void main() async {
  test('node defaults', () async {
    Node node = NodeImpl('name', 'owner','path');
    expect(node.visibility, Visibility.red,
        reason: 'test predefined owner');
  });

  test('node equality equals() after deep clone', () async {
    Node node = NodeImpl('name', 'owner','path');
    Node node2 = await node.deepClone();
    expect(node.equals(node2), true, reason: 'equality not given original:$node; clone:$node2');
  });

  test('ordinals mismatch', () async {
    Node node = NodeImpl('name', 'owner','path');
    Node node2 = await node.deepClone();
    node2.owner='newOwner';
    expect(node.equals(node2), false);
  });

  test('equality for child nodes', () async {
    Node node = NodeImpl('name', 'owner','path');
    Node node2 = await node.deepClone();
    await node2.addChild(NodeImpl('name2', 'path:name'));
    expect(node.equals(node2), false);
  });

  test('node serialization' , () async {
    Node node = NodeImpl('name', 'owner','path');
    ByteSink sink = ByteSink();
    node.toByteArrayStream(sink);
    sink.close();
    ByteStream stream = ByteStream(null,await sink.bytes);
    Node n2=await NodeImpl.fromByteArrayStream(stream);
    expect(n2.equals(node),isTrue);
  });
}
