library geiger_localstorage;

import 'dart:convert';

import 'package:geiger_localstorage/src/serialization/byte_sink.dart';
import 'package:geiger_localstorage/src/serialization/byte_stream.dart';
import 'package:geiger_localstorage/src/serialization/serializer_helper.dart';
import 'package:test/test.dart';

class SerializerHelperTest {}

void main() {
  group('Testing byte streams and sinks', () {
    test('sink test String', () async {
      for (String str in ['hello', 'öä\$üç&+"@']) {
        ByteSink s = ByteSink();
        s.sink.add(utf8.encode(str));
        s.close();
        expect(str, utf8.decode(await s.bytes), reason: 'strings not equal');
      }
    });
    test('sink test list<int>', () async {
      for (List<int> str in [
        [0],
        [-1, 1]
      ]) {
        ByteSink s = ByteSink();
        s.sink.add(str);
        s.close();
        expect(str, await s.bytes, reason: 'lists not equal');
      }
    });
  });
  group('primitive de/serialization tests', () {
    test('testing byte order serialization', () async {
      ByteSink sink = ByteSink();
      SerializerHelper.writeInt(sink, 257);
      sink.close();
      var l = await sink.bytes;
      expect(l[0],89, reason: 'uid byte 1 missmatch');
      expect(l[1],159, reason: 'uid byte 2 missmatch');
      expect(l[2],201, reason: 'uid byte 3 missmatch');
      expect(l[3],111, reason: 'uid byte 4 missmatch');
      expect(l[4],28, reason: 'uid byte 5 missmatch');
      expect(l[5],0, reason: 'uid byte 6 missmatch');
      expect(l[6],0, reason: 'uid byte 7 missmatch');
      expect(l[7],0, reason: 'uid byte 8 missmatch');
      expect(l[8],1, reason: 'data byte 1 missmatch');
      expect(l[9],1, reason: 'data byte 2 missmatch');
      expect(l[10],0, reason: 'data byte 3 missmatch');
      expect(l[11],0, reason: 'data byte 4 missmatch');
    });
    test('testing serialization/deserialization of int', () async {
      for (int s in [0, 1, -1, (1 << 31)-1]) {
        ByteSink sink = ByteSink();
        SerializerHelper.writeInt(sink, s);
        sink.close();
        var l = await sink.bytes;
        expect(12, l.length, reason: 'Length mismatch of stream');
        //print('## number=${s}');
        //print('## 0=${l[8]}');
        //print('## 1=${l[9]}');
        //print('## 2=${l[10]}');
        //print('## 3=${l[11]}');
        //print('');
        var s2 = await SerializerHelper.readInt(ByteStream(null, l));
        expect(s2, s, reason: 'deserialization of int failed for $s (resulted in $s2)');
      }
    });
    test('testing serialization/deserialization of long', () async {
      for (int s in [0, 1, -1, (1 << 63)-1]) {
        ByteSink sink = ByteSink();
        SerializerHelper.writeLong(sink, s);
        sink.close();
        var s2 =
            await SerializerHelper.readLong(ByteStream(null, await sink.bytes));
        expect(s2, s, reason: 'deserialization of int failed for $s');
      }
    });
    test('testing serialization/deserialization of string', () async {
      for (String s in ['string', 'testMessage öäü^']) {
        ByteSink sink = ByteSink();
        SerializerHelper.writeString(sink, s);
        sink.close();
        var s2 = await SerializerHelper.readString(
            ByteStream(null, await sink.bytes));
        expect(s2, s, reason: 'deserialization failed for string "$s"');
      }
    });
  });
}
