import 'dart:async';
import 'dart:io';

import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:test/test.dart';

class UserTest {}

Future<void> luongTests() async {
  test('20210929-test1', () async {
    StorageController geigerToolboxStorageController =
        GenericController('MI-Cyberrange', SqliteMapper('./dbFileName.sqlite'));
    await geigerToolboxStorageController.close();
    File('./dbFileName.sqlite').deleteSync();
  });
  test('20210929-test2', () async {
    // init
    StorageController geigerToolboxStorageController =
        GenericController('MI-Cyberrange', DummyMapper('testowner'));

    // sample code
    var newScore = 123;
    var currentUserLevel = 123;
    try {
      Node node =await  geigerToolboxStorageController.get(':score-node');
      // this is bad coding as it depends on a full transaction database state. Introduced addOrUpdateValue as drop in replacement.
      node.addValue(NodeValueImpl('score', '$newScore'));
      node.addValue(NodeValueImpl('level', '$currentUserLevel'));
      await geigerToolboxStorageController.update(node);
    } catch (e) {
      print('Exception while creating node');
      Node node = NodeImpl('score-node','testowner');
      await geigerToolboxStorageController.add(node);
      node.addValue(NodeValueImpl('score', '$newScore'));
      node.addValue(NodeValueImpl('level', '$currentUserLevel'));
      await geigerToolboxStorageController.update(node);
    }
  });
}

class Event {
  final EventType _event;
  final Node? _old;
  final Node? _new;

  Event(this._event, this._old, this._new);

  EventType get type => _event;

  Node? get oldNode => _old;

  Node? get newNode => _new;

  @override
  String toString() {
    return '${type.toString()} ${oldNode.toString()}=>${newNode.toString()}';
  }
}

class CustomStorageListener implements StorageListener {
  List<Event> events = <Event>[];

  @override
  void gotStorageChange(EventType event, Node? oldNode, Node? newNode) {
    Event e = Event(event, oldNode, newNode);
    events.add(e);
    print('got event ${e.toString()}');
  }

  Future<List<Event>> getNumEvents(int num,[int timeout=1000000]) async {
    int start =DateTime.now().millisecondsSinceEpoch;
    List<Event> ret= await Future.doWhile(() => events.length < num && start+1000*timeout>DateTime.now().millisecondsSinceEpoch).then((value) => events);
    if(events.length<num) {
      throw TimeoutException('Timeout reached while waiting for $num events');
    }
    return ret;
  }
}

void nosaTests() {
  test(
      '20211012-flutter Github Issue 16 (https://github.com/cyber-geiger/toolbox-storage/issues/16)',
      () async {
    StorageController _storageController =
        GenericController("Example", SqliteMapper("./database.db"));
    await _storageController.zap();
    CustomStorageListener l = CustomStorageListener();
    SearchCriteria s = SearchCriteria(':');
    _storageController.registerChangeListener(l, s);
    Node n = NodeImpl(':hello','testowner');
    expect(await s.evaluate(n),isTrue , reason: 'node fails to evaluate successfully');
    List<Event> e= await _storageController.add(n).then((_) async => await l.getNumEvents(1,2));
    expect(e.length,1, reason: 'Did not get expected number of results');
  });
}

Future<void> main() async {
  await luongTests();
  nosaTests();
}
